package tech.riemann.ims.controller.platform.material;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import org.nutz.lang.Strings;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import tech.riemann.ims.dto.request.ApplyInfo;
import tech.riemann.ims.entity.material.ApplyDetail;
import tech.riemann.ims.entity.material.ApplyForm;
import tech.riemann.ims.entity.material.Material;
import tech.riemann.ims.entity.material.MaterialStockDetail;
import tech.riemann.ims.enums.ApplyTypeEnum;
import tech.riemann.ims.enums.StockStatusEnum;
import tech.riemann.ims.service.material.IApplyDetailService;
import tech.riemann.ims.service.material.IApplyFormService;
import tech.riemann.ims.service.material.IMaterialService;
import tech.riemann.ims.service.material.IMaterialStockDetailService;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author mayong
 * @since 2024/11/28 15:29
 */
@RestController
@Tag(name = "Apply", description = "入库/出库 申请单管理")
@RequiredArgsConstructor
public class ApplyFormController {

    private final IApplyFormService applyFormService;
    private final IApplyDetailService applyDetailService;
    private final IMaterialStockDetailService materialStockDetailService;
    private final IMaterialService materialService;

    @GetMapping("apply")
    @Operation(summary = "分页查询申请单列表")
    public IPage<ApplyForm> applies(
            @Parameter(description = "页码") @RequestParam(required = false, defaultValue = "1") int page,
            @Parameter(description = "页面大小") @RequestParam(required = false, defaultValue = "10") int size,
            @Parameter(description = "类型") @RequestParam(name = "type") Integer type,  // 1入库 2出库
            @Parameter(description = "搜索关键词") @RequestParam(required = false, defaultValue = "") String key) {
        boolean hasLike = Strings.isNotBlank(key);
        key = String.format("%%%s%%", key);
        return applyFormService.page(Page.of(page, size), Wrappers.<ApplyForm>lambdaQuery()
                .eq(ApplyForm::getType, type)
                .orderByDesc(ApplyForm::getUpdatedTime));
    }

    @GetMapping("apply/{applyId}")
    @Operation(summary = "查询申请单详情")
    public ApplyInfo detail(@Parameter(description = "类型") @PathVariable(name = "applyId") Integer applyId) {
        List<ApplyDetail> list = applyDetailService.list(Wrappers.<ApplyDetail>lambdaQuery().eq(ApplyDetail::getApplyId, applyId));
        return ApplyInfo.builder()
               .applyForm(applyFormService.getById(applyId))
               .applyDetails(list)
               .build();
    }

    @PutMapping("apply")
    @Operation(summary = "提交申请单")
    @Transactional(rollbackFor = Exception.class)
    public void saveApply(@Validated @Parameter(description = "申请单") @RequestBody ApplyInfo applyInfo) {
        // 如果是采购入库 就新增数据，否则只改状态
        ApplyTypeEnum applyType = applyInfo.getApplyForm().getType();

        // 校验物料数量
        Map<Long, Integer> confirmMap = applyInfo.getApplyDetails().stream()
                .collect(Collectors.toMap(ApplyDetail::getMaterialId, ApplyDetail::getConfirmQuantity));
        Map<Long, Long> dataMap = applyInfo.getDetailList().stream()
                .collect(Collectors.groupingBy(MaterialStockDetail::getMaterialId, Collectors.counting()));
        confirmMap.forEach((k, v) -> {
            Long l = dataMap.get(k);
            if(l == null || l.intValue() != v){
                throw new IllegalArgumentException("["+k+"]物料数量不匹配");
            }
        });
        // 新增申请单
        applyFormService.save(applyInfo.getApplyForm());
        for (ApplyDetail applyDetail : applyInfo.getApplyDetails()) {
            applyDetail.setApplyId(applyInfo.getApplyForm().getId());
        }
        applyDetailService.saveBatch(applyInfo.getApplyDetails());

        if (applyType == ApplyTypeEnum.PURCHASE_RECEIPT) {
            // 新增物料
            applyInfo.getDetailList().forEach(detail -> {
                detail.setApplyId(applyInfo.getApplyForm().getId());
                detail.setStatus(StockStatusEnum.IN);
            });
            materialStockDetailService.saveBatch(applyInfo.getDetailList());
            //记录物料库存数据
            confirmMap.forEach((key, value) -> materialService.update(Wrappers.<Material>lambdaUpdate().set(Material::getStock, value)
                    .eq(Material::getId, key)));
        }
        else if(applyType == ApplyTypeEnum.LOAN_OUT || applyType == ApplyTypeEnum.RETURN_RECEIPT) {
            //修改物料状态
            applyInfo.getDetailList().forEach(detail -> materialStockDetailService.update(Wrappers.<MaterialStockDetail>lambdaUpdate()
                    .set(MaterialStockDetail::getMaterialId, applyType ==ApplyTypeEnum.RETURN_RECEIPT
                            ? StockStatusEnum.IN : StockStatusEnum.OUT)
                    .eq(MaterialStockDetail::getBarcode, detail.getBarcode())));
            //修改物料库存数据
            confirmMap.forEach((key, value) -> materialService.update(Wrappers.<Material>lambdaUpdate()
                    .setSql(applyType == ApplyTypeEnum.LOAN_OUT,"m_stock = m_stock - " + value)
                    .setSql(applyType == ApplyTypeEnum.RETURN_RECEIPT,"m_stock = m_stock + " + value)
                    .eq(Material::getId, key)));
        }
    }


}
